// MultiProgressBar.cpp : implementation file
//

#include "stdafx.h"
#include "MultiProgressBar.h"
#include "colors.h"

//#define _TRACE

// CMultiProgressBar

IMPLEMENT_DYNAMIC(CMultiProgressBar, CStatic)

CMultiProgressBar::CMultiProgressBar()
{

}

CMultiProgressBar::~CMultiProgressBar()
{
}


BEGIN_MESSAGE_MAP(CMultiProgressBar, CStatic)
    ON_WM_PAINT()
END_MESSAGE_MAP()



// CMultiProgressBar message handlers



/****************************************************************************
*                         CMultiProgressBar::SetRange
* Inputs:
*       int L: Low range
*       int H: High range
* Result: void
*       
* Effect: 
*       Sets the range
****************************************************************************/

void CMultiProgressBar::SetRange(int L, int H)
    {
     low = L;
     high = H;
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CMultiProgressBar::SetRange

/****************************************************************************
*                       CMultiProgressBar::SetSegments
* Inputs:
*       int n: Number of segments
* Result: void
*       
* Effect: 
*       Sets the number of segments
* Notes:
*       Requires that SetRange has already been done
****************************************************************************/

void CMultiProgressBar::SetSegments(int n)
    {
#ifdef _TRACE
     TRACE("CMultiProgressBar::SetSegments(%d)\n", n);
#endif
     if(n == 0 && limits.GetSize() == 0)
         return;
     if(n == 0)
         { /* clear it */
          limits.SetSize(0);
          return;
         } /* clear it */
     if(limits.GetSize() == n + 1)
        return; // already set
     limits.SetSize(n + 1);
     for(int i = 0; i < limits.GetSize(); i++)
        { /* initialize */
         limits[i].start = (int)(low + i * ((high - low) / (limits.GetSize() - 1)));
         limits[i].current = limits[i].start;
#ifdef _TRACE
         TRACE("limits[%d]=%d,%d\n", i, limits[i].start, limits[i].current);
#endif
        } /* initialize */
     if(GetSafeHwnd() != NULL)
        Invalidate();
    } // CMultiProgressBar::SetSegments

/****************************************************************************
*                    CMultiProgressBar::InvalidateSegment
* Inputs:
*       int n: Segment number
* Result: void
*       
* Effect: 
*       Invalidates the segment
****************************************************************************/

void CMultiProgressBar::InvalidateSegment(int n)
    {
     int left = limits[n].start;
     int right = limits[n+1].start;
     CRect r;
     GetClientRect(&r);
     r.left = left;
     r.right = right;
     CClientDC dc(this);
     PrepareDC(dc);
     dc.LPtoDP(&r);  // convert to device coordinates for invalidation
     InvalidateRect(&r);
#ifdef _TRACE
     TRACE("InvalidateSegment(%d): %d..%d\n", n, r.left, r.right);
#endif
    } // CMultiProgressBar::InvalidateSegment

/****************************************************************************
*                          CMultiProgressBar::SetPos
* Inputs:
*       int segment: Segment to set
*       int pos: Position to set
* Result: void
*       
* Effect: 
*       Sets the position for the segment
****************************************************************************/

void CMultiProgressBar::SetPos(int seg, int pos)
    {
#ifdef _TRACE
     TRACE("SetPos(%d, %d)\n", seg, pos);
#endif
     ASSERT(seg < limits.GetSize());
     if(seg >= limits.GetSize() - 1)
        return; // skip it

     limits[seg].current = pos;
     if(GetSafeHwnd())
        InvalidateSegment(seg);
    } // CMultiProgressBar::SetPos

/****************************************************************************
*                        CMultiProgressBar::PrepareDC
* Inputs:
*       CDC & dc:
* Result: void
*       
* Effect: 
*       Sets the mapping mode for the DC
****************************************************************************/

void CMultiProgressBar::PrepareDC(CDC & dc)
    {
     CRect r;
     GetClientRect(&r);
     
     dc.SetMapMode(MM_ANISOTROPIC);
     dc.SetWindowOrg(low, 0);
     dc.SetWindowExt((high - low), r.Height());

     dc.SetViewportOrg(0,0);
     dc.SetViewportExt(r.Width(), r.Height());
    } // CMultiProgressBar::PrepareDC

/****************************************************************************
*                         CMultiProgressBar::OnPaint
* Result: void
*       
* Effect: 
*       Paints the image
* Notes:
* Sample: Segments == 5
* Range = (0, 100)
* +--------------+--------------+--------------+--------------+--------------+
* |              |              |              |              |              |
* +--------------+--------------+--------------+--------------+--------------+
*
* 
****************************************************************************/

void CMultiProgressBar::OnPaint()
    {
     CPaintDC dc(this); // device context for painting

     if(limits.GetSize() == 0)
        { /* empty */
         CRect r;
         GetClientRect(&r);
         dc.FillSolidRect(&r, ::GetSysColor(COLOR_3DFACE));
         return;
        } /* empty */

     PrepareDC(dc);

     for(int i = 0; i < limits.GetSize() - 1; i++)
        { /* draw rectangle */
         CRect r;
         GetClientRect(&r);
         r.left = limits[i].start;
         r.right = limits[i].current;

#ifdef _TRACE
         TRACE("Paint seg [%d]=%d: %d..%d\n", i, limits[i].current, r.left, r.right);
#endif

         dc.FillSolidRect(&r, GetColor(i));
         r.left = limits[i].current;
         r.right = limits[i+1].start;

         dc.FillSolidRect(&r, ::GetSysColor(COLOR_3DFACE));
        } /* draw rectangle */
     // Do not call CStatic::OnPaint() for painting messages
    }
