// Plot.cpp : implementation file
//

#include "stdafx.h"
#include "Plot.h"
#include "colors.h"

// CPlot

IMPLEMENT_DYNAMIC(CPlot, CStatic)

CPlot::CPlot()
{
 firsttime = 0;  // the first time display we can do is at location 0
}

CPlot::~CPlot()
{
}


BEGIN_MESSAGE_MAP(CPlot, CStatic)
    ON_WM_PAINT()
END_MESSAGE_MAP()



// CPlot message handlers

/****************************************************************************
*                               CPlot::AddTick
* Inputs:
*       int section: Section #
*       int tn: Thread # (-1 for master thread)
*       DWORD t: Timestamp relative to 0
* Result: void
*       
* Effect: 
*       Adds a tick to the tick array
****************************************************************************/

void CPlot::AddTick(int section, int tn, DWORD t)
    {
     ticks.Add(TickInfo(section, tn, t));
     Invalidate();
    } // CPlot::AddTick

/****************************************************************************
*                              CPlot::ClearTicks
* Result: void
*       
* Effect: 
*       Clears the tick count
****************************************************************************/

void CPlot::ClearTicks()
    {
     ticks.SetSize(0);
     Invalidate();
    } // CPlot::ClearTicks

/****************************************************************************
*                               CPlot::AddTime
* Inputs:
*       double t: Time spent
*       UINT threads: number of threads
* Result: void
*       
* Effect: 
*       Adds the <threads, time> pair to the statistics
****************************************************************************/

void CPlot::AddTime(double t, UINT threads)
    {
     times.SetAtGrow(threads, data(t));
     
     firsttime = 0;  // assume value for count of 1 is valid for now; this may change
     double tmax = 0.0;
     tpoints.SetSize(0);
     for(int i = 0; i < times.GetSize(); i++)
        { /* scan times */
         double d = times[i].GetValue();
         tmax = max(tmax, d);
        } /* scan times */

#define TIMESCALE 1000
#define EPSILON 1.0E-6
     if(tmax < EPSILON)
        return;  // can't handle tiny values   

     // Normalize the values to the range 0..TIMESCALE
     // We need a
     int runstart = 0;
     int runend = 0;
     
     for(int i = 1; i < times.GetSize(); i++)
        { /* convert times */
         double d = times[i].GetValue();
         if(d < 0.0)
            { /* interpolate */
             // Note that we cannot have a -1 value in the last place
             // because only valid values form the high-water mark
             // so we know if we have a -1 there is at least one
             // value beyond this one
             //

             if(runstart == 0)
                { /* start interpoltion */
                 runstart = runend = i;
#ifdef _TRACE
                 TRACE("Start interpolation: %d\n", i);
#endif
                } /* start interpoltion */
             else
                { /* continue interpolation */
#ifdef _TRACE
                 TRACE("Continue interpolation: %d\n", i);
#endif
                } /* continue interpolation */
             continue;
            } /* interpolate */

         // We get here, we could have a run
         // for which we must interpolate

         if(runstart == 1)
            { /* leading run */
             //****************************************************************
             // times
             //    0   1   2   3   4   5   6   7   8   9
             //  +---+---+---+---+---+---+---+---+---+---+
             //  |###|###|###|###| v |###|###|###|###| v |
             //  +---+---+---+---+---+---+---+---+---+---+
             //        ^           ^
             //        |           |
             //        |           i
             //        runstart
             //****************************************************************
             CPoint pt(0,0);
             firsttime = i;
#ifdef _TRACE
             TRACE("firsttime = %d\n", firsttime);
#endif
             for(int k = runstart; k < i; k++)
                tpoints.Add(pt); // keep indices consistent
             //****************************************************************
             //    0   1   2   3   4   5   6   7   8   9
             //  +---+---+---+---+---+---+---+---+---+---+
             //  |###|###|###|###| v |###|###|###|###| v |
             //  +---+---+---+---+---+---+---+---+---+---+
             // tpoints
             //        0   1   2   3  
             //      +---+---+---+---
             //      | - | - | - | ?
             //      +---+---+---+---
             //                    ^
             //                    |
             //                    firsttime (value not yet filled in)
             //****************************************************************
             runstart = runend = 0; // get ready for next run
            } /* leading run */
         else
         if(runstart != 0)
            { /* do interpolation */
             //****************************************************************
             //
             //                    *----------------------------
             //                                                ^
             //                                                |span
             //                                                |
             //                                                v
             //                                        *--------            
             //    0   1   2   3   4   5   6   7   8   9
             //  +---+---+---+---+---+---+---+---+---+---+
             //  |###|###|###|###| v |###|###|###|###| v |
             //  +---+---+---+---+---+---+---+---+---+---+
             //                    ^   ^           ^
             //                    |   |           |
             //            firsttime   |           runend
             //                        runstart
             //****************************************************************
             runend = i - 1;
#ifdef _TRACE
             TRACE("End interpolation %d\n", i);
#endif
             double span = times[runstart - 1].GetValue() - times[runend + 1].GetValue();
             double delta = span / (double)(runend - runstart + 2);
#ifdef _TRACE
             TRACE("    span = [%d]=%9.6f, [%d]=%9.6f =>%9.6f\n", runstart - 1, times[runstart - 1].GetValue(),
                                                                  runend + 1,   times[runend + 1].GetValue(),
                                                                  span);
             TRACE("    delta = %9.6f/(%d - %d + 2) = %9.6f/%d = %9.6f\n", span, runend, runstart,
                                                                       span, (runend - runstart) + 2,
                                                                       delta);
#endif             
             for(int k = runstart; k <= runend; k++)
                { /* comput interpolated point */
                 double d = (times[runstart - 1].GetValue() - delta * (double)(k - runstart + 1));
#ifdef _TRACE
                 TRACE("    [%d] = %9.6f\n", k, d);
#endif
                 CPoint pt;
                 pt.x = k;
                 pt.y = (int)((double)TIMESCALE * (d / tmax));
                 tpoints.Add(pt);
                } /* compute interpolated point */
             //****************************************************************
             //                  |
             //                  v
             //                -----*----------------------------
             //                --------+                       ^
             //                  ^         +                   |span
             //                  |             +               |
             //              delta                 +           v
             //                                        *--------            
             //    0   1   2   3   4   5   6   7   8   9
             //  +---+---+---+---+---+---+---+---+---+---+
             //  |###|###|###|###| v |###|###|###|###| v |
             //  +---+---+---+---+---+---+---+---+---+---+
             //                    ^   ^           ^
             //                    |   |           |
             //            firsttime   |           runend
             //                        runstart
             //****************************************************************
             runstart = runend = 0;
            } /* do interpolation */

         CPoint pt;
         pt.x = i;
         pt.y = (int)((double)TIMESCALE * (d / tmax));
         tpoints.Add(pt);
        } /* convert times */

     Invalidate();
    } // CPlot::AddTime

/****************************************************************************
*                              CPlot::ClearTimes
* Result: void
*       
* Effect: 
*       Clears the times plot
****************************************************************************/

void CPlot::ClearTimes()
    {
     times.SetSize(0);
     tpoints.SetSize(0);
     points.SetSize(0);
     ticks.SetSize(0);
     Invalidate();
    } // CPlot::ClearTimes


/****************************************************************************
*                              CPlot::SetPoints
* Inputs:
*       CArray<LARGE_INTEGER, LARGE_INTEGER> & times
* Result: void
*       
* Effect: 
*       Sets the points
****************************************************************************/

void CPlot::SetPoints(CArray<LARGE_INTEGER, LARGE_INTEGER> & times)
    {
     points.SetSize(times.GetSize());

     LARGE_INTEGER freq;
     ::QueryPerformanceFrequency(&freq);
     double f = (double)freq.QuadPart;

     double m = 0.0;
     for(int i = 0; i < times.GetSize(); i++)
        { /* normalize times */
         double t = (double)times[i].QuadPart / f;
         m = max(t, m);
        } /* normalize times */
     // m is now the maximum time, representing 100%
     // so any time t < m is (t/m) of the ratio

     polypoints.SetSize(0);
     ymax = 0;
     int lastpoint = 0;
     for(int i = 0; i < times.GetSize(); i++)
        { /* plot times */
         double t = ((double)times[i].QuadPart / f) / m;
         points[i].x = i;
         points[i].y = (int) (t * 10000.0);
         if(i > 0 && points[i].y < points[i - 1].y)
            { /* segment boundary */
             // Now we need to compute the PolyPolyline array.
             // Since each segment is monotonically increasing, the
             // boundaries are the place where the slope changes
             polypoints.Add((DWORD)(i - lastpoint));
             lastpoint = i;
            } /* segment boundary */
#ifdef _DEBUG
         CPoint pt(points[i]); 
#endif
         ymax = max(ymax, points[i].y);
        } /* plot times */
     polypoints.Add((DWORD)((int)times.GetSize() - lastpoint));


     Invalidate();
    } // CPlot::SetPoints


/****************************************************************************
*                               CPlot::OnPaint
* Result: void
*       
* Effect: 
*       Plots the event vs. time plot
****************************************************************************/

void CPlot::OnPaint()
    {
     CPaintDC dc(this); // device context for painting

     int saveclip = dc.SaveDC();
     CRgn rgn;
     CRect r;
     GetClientRect(&r);
     rgn.CreateRectRgn(r.left, r.top, r.right, r.bottom);
     dc.SelectClipRgn(&rgn, RGN_AND);

     //****************************************************************
     // Fill in the background
     //****************************************************************
     if(points.GetSize() == 0 && ticks.GetSize() == 0)
        { /* no times */
         CRect r;
         GetClientRect(&r);
         dc.FillSolidRect(&r, ::GetSysColor(COLOR_3DFACE));
        } /* no times */
     else
        { /* has times */
         CRect r;
         GetClientRect(&r);
         dc.FillSolidRect(&r, RGB(255, 128, 192));
        } /* has times */

     

     //****************************************************************
     // Plot the x=nthreads, y = cumulative time
     //****************************************************************

     if(firsttime < tpoints.GetSize())
        { /* plot points */
         CRect r;
         GetClientRect(&r);
         
         int save = dc.SaveDC();

         dc.SetMapMode(MM_ANISOTROPIC);

         dc.SetWindowOrg(0, TIMESCALE);
         dc.SetWindowExt((int)(tpoints.GetSize()), - TIMESCALE);

         dc.SetViewportOrg(0,0);
         dc.SetViewportExt(r.Width(), r.Height());

         //****************************************************************
         //    0   1   2   3   4   5   6   7   8   9
         //  +---+---+---+---+---+---+---+---+---+---+
         //  |###|###|###|###| v |###|###|###|###| v |
         //  +---+---+---+---+---+---+---+---+---+---+
         //      :             ^                     :
         //      :             |                     :
         //      :     firsttime                     :
         //      :                                   :
         //      :           |                       :
         //      :           v                       :
         //      :         -----*----------------------------
         //      :         --------+                 :     ^
         //      :           ^         +             :     |span
         //      :           |             +         :     |
         //  tpoints         delta             +     :     v
         //      :                                 *--------            
         //      : 0   1   2   3   4   5   6   7   8 : 
         //      +---+---+---+---+---+---+---+---+---+
         //      | - | - | - | v | * | * | * | * | v |
         //      +---+---+---+---+---+---+---+---+---+
         //                  : ^                   ^ :
         //                  : |                   | :
         //          firsttime-1                   tpoints.GetSize()-1
         //                  :                       :
         //                  :                       :
         //                  +---+---+---+---+---+---+
         //       xform:     | v | * | * | * | * | v |
         //                  +---+---+---+---+---+---+
         //****************************************************************

         CArray<CPoint, CPoint> xform;
         { /* copy values */
          int start;
          if(firsttime == 0)
              start = 0;
          else 
              start = firsttime - 1;
          for(int i = start; i < tpoints.GetSize(); i++)
               xform.Add(tpoints[i]);
         } /* copy values */

         //****************************************************************
         // Now transform the logical space to device space so we can
         // do the rest of our drawing device space
         //****************************************************************

         dc.LPtoDP(xform.GetData(), (int)xform.GetSize());
         
         //****************************************************************
         // Fiducial lines for plot counts
         //****************************************************************
         CPen lines(PS_DOT, 0, RGB(255, 0, 0));
         dc.SelectObject(&lines);

         for(int i = 0; i < tpoints.GetSize(); i++)
            { /* plot lines */
             dc.MoveTo(i, 0);
             dc.LineTo(i, TIMESCALE);
            } /* plot lines */

         dc.RestoreDC(save);

         //****************************************************************
         // Now draw the line, in device space
         //****************************************************************

         save = dc.SaveDC();

         CPen pen(PS_DASH, 0, RGB(0, 0, 255));

         dc.SelectObject(&pen);
         dc.Polyline(xform.GetData(), (int)xform.GetSize());
         
         // Now plot the little mark to indicate a block
#define BLOCK 3
         dc.SelectStockObject(BLACK_BRUSH);
         dc.SelectStockObject(BLACK_PEN);

         //****************************************************************
         //    0   1   2   3   4   5   6   7   8   9
         //  +---+---+---+---+---+---+---+---+---+---+
         //  |###|###|###|###| v |###|###|###|###| v |
         //  +---+---+---+---+---+---+---+---+---+---+
         //      :             ^                     :
         //      :             |                     :
         //      :     firsttime                     :
         //      :                                   :
         //      :           |                       :
         //      :           v                       :
         //      :         -----*----------------------------
         //      :         --------+                 :     ^
         //      :           ^         +             :     |span
         //      :           |             +         :     |
         //  tpoints         delta             +     :     v
         //      :                                 *--------            
         //      : 0   1   2   3   4   5   6   7   8 : 
         //      +---+---+---+---+---+---+---+---+---+
         //      | - | - | - | v | * | * | * | * | v |
         //      +---+---+---+---+---+---+---+---+---+
         //                  : ^                   ^ :
         //                  : |                   | :
         //          firsttime-1                   tpoints.GetSize()-1
         //                  :                       :
         //                  :                       :
         //                  +---+---+---+---+---+---+
         //       xform:     | v | * | * | * | * | v |
         //                  +---+---+---+---+---+---+
         //****************************************************************
         { /* draw points */
          int start = firsttime - 1;
          if(firsttime == 0)
              start = 0;
          for(int i = start; i < tpoints.GetSize(); i++)
             { /* scan values */
              if(times[i + 1].GetValue() >= 0.0)
                 { /* show block */
                  int n = i - start;
                  CRect r(xform[n].x - BLOCK, xform[n].y - BLOCK,
                          xform[n].x + BLOCK, xform[n].y + BLOCK);
                  dc.Rectangle(r);
                 } /* show block */
             } /* scan values */
         } /* draw points */
         dc.RestoreDC(save);
        } /* plot points */

     //****************************************************************
     // Plot the timestamp plot
     //****************************************************************
     // This typically produces a sawtooth pattern
     // +----------------------------+
     // |     /      /      /      / |
     // |    /      /      /      /  |
     // |   /      /      /      /   |
     // |  /      /      /      /    |
     // | /      /      /      /     |
     // |/      /      /      /      |
     // +----------------------------+

     if(points.GetSize() > 1)
        { /* plot sawtooth */
         CRect r;
         GetClientRect(&r);

         int save = dc.SaveDC();

         dc.SetMapMode(MM_ANISOTROPIC);

         dc.SetWindowOrg(0, ymax);
         dc.SetWindowExt((int)points.GetSize(), - ymax);

         dc.SetViewportOrg(0,0);
         dc.SetViewportExt(r.Width(), r.Height());

         //dc.Polyline(points.GetData(), (int)points.GetSize());
         dc.PolyPolyline(points.GetData(), polypoints.GetData(), (int)polypoints.GetSize());

         dc.RestoreDC(save);
        } /* plot sawtooth */

     //****************************************************************
     // Plot tick points
     //****************************************************************
     if(ticks.GetSize() > 0)
        { /* plot ticks */
         int save = dc.SaveDC();

         DWORD maxtime = 0;
         DWORD mintime = UINT_MAX;
         int minthread = INT_MAX;
         int maxthread = INT_MIN;

         for(int i = 0; i < ticks.GetSize(); i++)
            { /* scan limits */
             DWORD t = ticks[i].GetTime();
             maxtime = max(maxtime, t);
             mintime = min(mintime, t);

             int tn = ticks[i].GetThreadNo();
             maxthread = max(maxthread, tn);
             minthread  = min(minthread, tn);
            } /* scan limits */

         dc.SetMapMode(MM_ANISOTROPIC);

         CPoint org(mintime, maxthread + 1);
         dc.SetWindowOrg(org.x, org.y);
#ifdef _TRACE
         TRACE("Sections: SetWindowOrg(%d, %d)\n", org.x, org.y);
#endif

         CSize ext(max(maxtime, 1), - (maxthread - minthread + 1 + 1));
         dc.SetWindowExt(ext.cx, ext.cy);
#ifdef _TRACE
         TRACE("Sections: SetWindowExt(%d, %d)\n", ext.cx, ext.cy);
#endif
         dc.SetViewportOrg(0,0);
         dc.SetViewportExt(r.Width(), r.Height());

         CArray<CPoint,CPoint> pts;
         pts.SetSize(ticks.GetSize());
         CArray<COLORREF, COLORREF> ptcolors;
         ptcolors.SetSize(ticks.GetSize());

         for(int i = 0; i < ticks.GetSize(); i++)
            { /* plot points */
             pts[i].x = ticks[i].GetTime();
             pts[i].y = ticks[i].GetThreadNo();
#ifdef _TRACE
             TRACE("Sections: pts[%d] = (%d, %d)\n", i, pts[i].x, pts[i].y);
#endif                                        
             dc.LPtoDP(&pts[i]);
             ptcolors[i] = GetColor(ticks[i].GetSection());
#ifdef _TRACE
             TRACE("          pts[%d] = (%d, %d) RGB(%d, %d, %d)\n",
                                 i,
                                 pts[i].x, pts[i].y,
                                 GetRValue(ptcolors[i]),
                                 GetGValue(ptcolors[i]),
                                 GetBValue(ptcolors[i]));
#endif
            } /* plot points */
         dc.RestoreDC(save);

         // Now draw the points in device space
         for(int i = 0; i < pts.GetSize(); i++)
            { /* draw each */
             CRect r(pts[i].x - BLOCK, pts[i].y - BLOCK,
                     pts[i].x + BLOCK, pts[i].y + BLOCK);
             dc.FillSolidRect(&r, ptcolors[i]);
            } /* draw each */
        } /* plot ticks */

     dc.RestoreDC(saveclip);
     // Do not call CStatic::OnPaint() for painting messages
    }
