// parallelfor.cpp: Separated out the code for the threading support so it can be
//             easily studied (shorter .cod file) or modified

#include "stdafx.h"
#include "openmpdlg.h"
#include "Messages.h"
#include <omp.h>
#include "IteratorLimit.h"

/****************************************************************************
*                            CopenmpDlg::WasteTime
* Inputs:
*       int w:
* Result: void
*       
* Effect: 
*       Does nothing
****************************************************************************/
#pragma optimize("g", off)
__declspec(noinline) void CopenmpDlg::WasteTime(int w)
    {
     return;
    } // CopenmpDlg::WasteTime

/****************************************************************************
*                            CopenmpDlg::DoNothing
* Inputs:
*       int i: Value
* Result: void
*       
* Effect: 
*       Uses up some time
****************************************************************************/

__declspec(noinline) void CopenmpDlg::DoNothing(int i)
    {
     int w = i * i;
     WasteTime(w);
    } // CopenmpDlg::DoNothing


/****************************************************************************
*                           CopenmpDlg::DoSomething
* Inputs:
*       int i: Value to process
* Result: void
*       
* Effect: 
*       Executes a couple operations that send messagesw to the parent
*       It aslo records the current QueryPerformanceCounter time
* Notes:
*       Because of processor time skew, the QueryPerformanceCounter values
*       are not exactly precise, but they appear to be close enough.
****************************************************************************/

void CopenmpDlg::DoSomething(int i)
    {
     ::QueryPerformanceCounter(&times[i]);
     int numthreads = omp_get_num_threads();
#ifdef _TRACE
     TRACE("%08x: Sending UWM_SET_THREADCOUNT(%d)\n", ::GetCurrentThreadId(), numthreads);
#endif

     QueueMessage(m_hWnd, UWM_SET_THREADCOUNT, (WPARAM)numthreads);

     int tn = omp_get_thread_num();
#ifdef _TRACE
     TRACE("%08x: Sending UWM_THREAD_INFO(%d, %d)\n", ::GetCurrentThreadId(), tn, i);
#endif
     QueueMessage(m_hWnd, UWM_THREAD_INFO, (WPARAM)tn, (WPARAM)i);

     for(int i = 0; i < work; i++)
        DoNothing(i);
    } // CopenmpDlg::DoSomething


/****************************************************************************
*                          CopenmpDlg::DoForAllLoop
* Inputs:
*       int nthreads: Number of threads to report
* Result: void
*       
* Effect: 
*       Runs the loop
****************************************************************************/

void CopenmpDlg::DoForAllLoop(int nthreads)
   {
//*************************************************************//*
// the loop                                                    //*
//*************************************************************//*
    LARGE_INTEGER start;                                       //*
    ::QueryPerformanceCounter(&start);                         //*
                                                               //*
#pragma omp parallel for schedule(runtime)                     //*
    for(int i = 0; i < ITERATOR_LIMIT; i++)                    //*
       DoSomething(i);                                         //*
                                                               //*
    LARGE_INTEGER end;                                         //*
    ::QueryPerformanceCounter(&end);                           //*
//*************************************************************//*
// end of loop                                                 //*
//*************************************************************//*

    LARGE_INTEGER duration;
    duration.QuadPart = end.QuadPart - start.QuadPart;
    ShowDuration(duration);
    LARGE_INTEGER freq;
    ::QueryPerformanceFrequency(&freq);

     // Normalize the times so they are delta-T from start
    for(int i = 0; i < times.GetSize(); i++)
       times[i].QuadPart -= start.QuadPart;

    c_Plot.SetPoints(times);

    ShowElapsedTime(duration, nthreads);

     
   } // CopenmpDlg::DoForAllLoop
