#include "stdafx.h"
#include "openmpdlg.h"
#include <omp.h>
#include "Messages.h"

/****************************************************************************
*                           CopenmpDlg::DoSectionsNoWait
* Inputs:
*       int delay: Delay value
* Result: void
*       
* Effect: 
*       Executes 5 parallel sections
****************************************************************************/

void CopenmpDlg::DoSectionsNoWait(const int delay)
    {
#if 0 // This doesn't work because the nowait is not accepted by the
      // Microsoft compiler, even though the official docs say it should be
     DoTick(-1, -1, delay, 10);
#pragma omp parallel sections  default(shared) nowait
      { /* sections */
#pragma omp section 
        { /* section 1 */ 
         DoTick(1, omp_get_thread_num(), delay, 10); 
        } /* section 1 */ 
#pragma omp section  
        { /* section 2 */ 
         DoTick(2, omp_get_thread_num(), delay, 12); 
        } /* section 2 */ 
#pragma omp section  
        { /* section 3 */ 
         DoTick(3, omp_get_thread_num(), delay, 8); 
        } /* section 3 */ 
#pragma omp section 
        { /* section 4 */ 
         DoTick(4, omp_get_thread_num(), delay, 9); 
        } /* section 4 */ 
#pragma omp section 
        { /* section 5 */ 
         DoTick(5, omp_get_thread_num(), delay, 11); 
        } /* section 5 */

      } /* sections */
      DoTick(-1, -1, delay, 5);
#endif // Currently not implemented
    } // CopenmpDlg::DoSectionsNoWait

/****************************************************************************
*                           CopenmpDlg::DoSections
* Inputs:
*       int delay: Delay value
* Result: void
*       
* Effect: 
*       Executes 5 parallel sections
****************************************************************************/

void CopenmpDlg::DoSections(const int delay)
    {
     DoTick(-1, -1, delay, 10);
#pragma omp parallel sections firstprivate(delay)
      { /* sections */
#pragma omp section 
        { /* section 1 */ 
         DoTick(1, omp_get_thread_num(), delay, 10); 
        } /* section 1 */ 
#pragma omp section  
        { /* section 2 */ 
         DoTick(2, omp_get_thread_num(), delay, 12); 
        } /* section 2 */ 
#pragma omp section  
        { /* section 3 */ 
         DoTick(3, omp_get_thread_num(), delay, 8); 
        } /* section 3 */ 
#pragma omp section 
        { /* section 4 */ 
         DoTick(4, omp_get_thread_num(), delay, 9); 
        } /* section 4 */ 
#pragma omp section 
        { /* section 5 */ 
         DoTick(5, omp_get_thread_num(), delay, 11); 
        } /* section 5 */
#pragma omp section 
        { /* section 6 */ 
         DoTick(6, omp_get_thread_num(), delay, 8); 
        } /* section 6 */
#pragma omp section 
        { /* section 7 */ 
         DoTick(7, omp_get_thread_num(), delay, 6); 
        } /* section 7 */
#pragma omp section 
        { /* section 8 */ 
         DoTick(8, omp_get_thread_num(), delay, 9); 
        } /* section 8 */

      } /* sections */
      DoTick(-1, -1, delay, 5);
    } // CopenmpDlg::DoSections

/****************************************************************************
*                             CopenmpDlg::DoTick
* Inputs:
*       const int section: Section #
*       const int tn: Thread #
*       const int delay: Tick delay
*       const int count: Iteration count
* Result: void
*       
* Effect: 
*       Loops and delays
****************************************************************************/

void CopenmpDlg::DoTick(const int section, const int tn, const int delay, const int count)
    {
     for(int i = 0; i < count; i++)
        { /* loop */
         MarkTime(section, tn);
         switch(WaitType)
            { /* WaitType */
             default:
                ASSERT(FALSE);
                // FALLS THRU
             case IDC_SLEEP:
                Sleep(delay);
                break;
             case IDC_SPIN:
#ifdef _DEBUG
#define WORKBIAS 10
#else
#define WORKBIAS 500
#endif
                for(int i = 0; i < work * WORKBIAS; i++)
                   DoNothing(i);
                break;
            } /* WaitType */
        } /* loop */
     MarkTime(section, tn);
    } // CopenmpDlg::DoTick

/****************************************************************************
*                            CopenmpDlg::MarkTime
* Inputs:
*       int section: Section #
*       int tn: Thread #
* Result: void
*       
* Effect: 
*       Sends a realtime-stamped notice
****************************************************************************/

void CopenmpDlg::MarkTime(int section, int tn)
    {
     DWORD t = ::GetTickCount();
     QueueMessage(m_hWnd, UWM_TICK, MAKEWPARAM(tn, section), (LPARAM)(t - startTime));
    } // CopenmpDlg::MarkTime

/****************************************************************************
*                          CopenmpDlg::DoSectionLoop
* Inputs:
*       int nthreads: Thread notification value
* Result: void
*       
* Effect: 
*       Rusn the section loop
****************************************************************************/

void CopenmpDlg::DoSectionLoop(int nthreads)
    {
     //****************************************************************//*
     // The loop                                                       //*
     //****************************************************************//*
     LARGE_INTEGER start;                                              //*
     ::QueryPerformanceCounter(&start);                                //*
     if(NoWait)                                                        //*
        { /* nowait */                                                 //*
         DoSectionsNoWait(delay);                                      //*
        } /* nowait */                                                 //*
     else                                                              //*
        { /* wait */                                                   //*
         DoSections(delay);                                            //*
        } /* wait */                                                   //*
     LARGE_INTEGER end;                                                //*
     ::QueryPerformanceCounter(&end);                                  //*
     //****************************************************************//*
     // end of loop                                                    //*
     //****************************************************************//*
     LARGE_INTEGER duration;
     duration.QuadPart = end.QuadPart - start.QuadPart;

     ShowDuration(duration);

     ShowElapsedTime(duration, nthreads);
    } // CopenmpDlg::DoSectionLoop
